#include "twn_rendering_c.h"
#include "twn_engine_context_c.h"
#include "twn_textures_c.h"

#include <stb_ds.h>


/* TODO: automatic handling of repeating textures */
/*       for that we could allocate a loner texture */
void unfurl_triangle(const char *path,
                     t_fvec3 v0,
                     t_fvec3 v1,
                     t_fvec3 v2,
                     t_shvec2 uv0,
                     t_shvec2 uv1,
                     t_shvec2 uv2)
{
    const t_texture_key texture_key = textures_get_key(&ctx.texture_cache, path);

    struct mesh_batch_item *batch_p = hmgetp_null(ctx.uncolored_mesh_batches, texture_key);
    if (!batch_p) {
        struct mesh_batch item = {0};
        hmput(ctx.uncolored_mesh_batches, texture_key, item);
        batch_p = &ctx.uncolored_mesh_batches[hmlenu(ctx.uncolored_mesh_batches) - 1]; /* TODO: can last index be used? */
    }

    union uncolored_space_triangle triangle = { .primitive = {
        .v0 = v0,
        .v1 = v1,
        .v2 = v2,
        .uv1 = m_to_fvec2(uv1),
        .uv0 = m_to_fvec2(uv0),
        .uv2 = m_to_fvec2(uv2),
    }};

    union uncolored_space_triangle *triangles = (union uncolored_space_triangle *)batch_p->value.primitives;

    arrpush(triangles, triangle);
    batch_p->value.primitives = (uint8_t *)triangles;
}


void draw_uncolored_space_traingle_batch(struct mesh_batch *batch,
                                         t_texture_key texture_key)
{
    static vertex_buffer vertex_array = 0;
    if (vertex_array == 0)
        vertex_array = create_vertex_buffer();

    const size_t primitives_len = arrlenu(batch->primitives);

    /* nothing to do */
    if (primitives_len == 0)
        return;

    const t_frect srcrect = textures_get_srcrect(&ctx.texture_cache, texture_key);
    const t_frect dims    = textures_get_dims(&ctx.texture_cache, texture_key);

    const float wr = srcrect.w / dims.w;
    const float hr = srcrect.h / dims.h;
    const float xr = srcrect.x / dims.w;
    const float yr = srcrect.y / dims.h;

    /* update pixel-based uvs to correspond with texture atlases */
    for (size_t i = 0; i < primitives_len; ++i) {
        struct uncolored_space_triangle_payload *payload =
            &((union uncolored_space_triangle *)batch->primitives)[i].payload;

        payload->uv0.x = xr + ((float)payload->uv0.x / srcrect.w) * wr;
        payload->uv0.y = yr + ((float)payload->uv0.y / srcrect.h) * hr;
        payload->uv1.x = xr + ((float)payload->uv1.x / srcrect.w) * wr;
        payload->uv1.y = yr + ((float)payload->uv1.y / srcrect.h) * hr;
        payload->uv2.x = xr + ((float)payload->uv2.x / srcrect.w) * wr;
        payload->uv2.y = yr + ((float)payload->uv2.y / srcrect.h) * hr;
    }

    specify_vertex_buffer(vertex_array, batch->primitives, primitives_len * sizeof (struct uncolored_space_triangle_payload));

    finally_draw_uncolored_space_traingle_batch(batch, texture_key, vertex_array);
}
